"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DetectionIntegrationTest = void 0;
const loader_1 = require("../motion/coco/loader");
const image_utils_1 = require("../utils/image-utils");
const assert = require("assert");
const path = require('path');
const fs = require('fs');
class DetectionIntegrationTest {
    constructor() {
    }
    async run() {
        console.log('Loader-detect-image-full-model-IT');
        let modelLoader = new loader_1.Loader(this.mockLogging());
        let detector = await modelLoader.loadCoco(true);
        await this.verifyDetections(detector);
        console.log('Loader-detect-image-lite-model-IT');
        modelLoader = new loader_1.Loader(this.mockLogging());
        detector = await modelLoader.loadCoco();
        await this.verifyDetections(detector);
    }
    async verifyDetections(detector) {
        assert(detector !== null);
        const homebridgeDir = require('os').homedir() + '/.homebridge';
        image_utils_1.ImageUtils.userStoragePath = homebridgeDir;
        if (!fs.existsSync(homebridgeDir)) {
            fs.mkdirSync(homebridgeDir);
        }
        let image = await image_utils_1.ImageUtils.createImage(path.resolve('../../resources/images/test.jpg'));
        assert(image !== null);
        let detections = await detector.detect(image, true);
        assert(detections !== null);
        assert(detections.length > 0);
        const annotatedImage = await image_utils_1.ImageUtils.generateAnnotatedImage(image, detections);
        const fileName = await image_utils_1.ImageUtils.saveCanvasToFile(annotatedImage);
        let stats = fs.statSync(fileName);
        assert(stats.isFile() == true);
        fs.unlinkSync(fileName);
    }
    ;
    mockLogging() {
        return {
            info: ((message) => {
                console.log(message);
            }),
            debug: ((message) => {
                console.log(message);
            })
        };
    }
}
exports.DetectionIntegrationTest = DetectionIntegrationTest;
(async () => {
    const test = new DetectionIntegrationTest();
    await test.run();
})();
//# sourceMappingURL=detection-integration-test.js.map