"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectDetection = exports.load = void 0;
const tfconv = __importStar(require("@tensorflow/tfjs-converter"));
const tf = __importStar(require("@tensorflow/tfjs-node"));
const classes_1 = require("./classes");
const BASE_PATH = 'https://storage.googleapis.com/tfjs-models/savedmodel/';
async function load(config = {}) {
    if (tf == null) {
        throw new Error('TensorFlow not loaded correctly!');
    }
    const base = config.base || 'lite_mobilenet_v2';
    const modelUrl = config.modelUrl;
    if (['mobilenet_v1', 'mobilenet_v2', 'lite_mobilenet_v2'].indexOf(base) ===
        -1) {
        throw new Error(`ObjectDetection constructed with invalid base model ` +
            `${base}. Valid names are 'mobilenet_v1',` +
            ` 'mobilenet_v2' and 'lite_mobilenet_v2'.`);
    }
    const objectDetection = new ObjectDetection(base, modelUrl);
    await objectDetection.load();
    return objectDetection;
}
exports.load = load;
class ObjectDetection {
    constructor(base, modelUrl) {
        this.modelPath =
            modelUrl || `${BASE_PATH}${this.getPrefix(base)}/model.json`;
    }
    getPrefix(base) {
        return base === 'lite_mobilenet_v2' ? `ssd${base}` : `ssd_${base}`;
    }
    async load() {
        this.model = await tfconv.loadGraphModel(this.modelPath);
        const zeroTensor = tf.zeros([1, 300, 300, 3], 'int32');
        const result = await this.model.executeAsync(zeroTensor);
        await Promise.all(result.map(t => t.data()));
        result.map(t => t.dispose());
        zeroTensor.dispose();
    }
    async infer(img, maxNumBoxes, minScore) {
        const batched = tf.tidy(() => {
            if (!(img instanceof tf.Tensor)) {
                img = tf.browser.fromPixels(img);
            }
            return tf.expandDims(img);
        });
        const height = batched.shape[1];
        const width = batched.shape[2];
        const result = await this.model.executeAsync(batched);
        const scores = result[0].dataSync();
        const boxes = result[1].dataSync();
        batched.dispose();
        tf.dispose(result);
        const [maxScores, classes] = this.calculateMaxScores(scores, result[0].shape[1], result[0].shape[2]);
        const prevBackend = tf.getBackend();
        if (tf.getBackend() === 'webgl') {
            tf.setBackend('cpu');
        }
        const indexTensor = tf.tidy(() => {
            const boxes2 = tf.tensor2d(boxes, [result[1].shape[1], result[1].shape[3]]);
            return tf.image.nonMaxSuppression(boxes2, maxScores, maxNumBoxes, minScore, minScore);
        });
        const indexes = indexTensor.dataSync();
        indexTensor.dispose();
        if (prevBackend !== tf.getBackend()) {
            tf.setBackend(prevBackend);
        }
        return this.buildDetectedObjects(width, height, boxes, maxScores, indexes, classes);
    }
    buildDetectedObjects(width, height, boxes, scores, indexes, classes) {
        const count = indexes.length;
        const objects = [];
        for (let i = 0; i < count; i++) {
            const bbox = [];
            for (let j = 0; j < 4; j++) {
                bbox[j] = boxes[indexes[i] * 4 + j];
            }
            const minY = bbox[0] * height;
            const minX = bbox[1] * width;
            const maxY = bbox[2] * height;
            const maxX = bbox[3] * width;
            bbox[0] = minX;
            bbox[1] = minY;
            bbox[2] = maxX - minX;
            bbox[3] = maxY - minY;
            objects.push({
                bbox: bbox,
                class: classes_1.CLASSES[classes[indexes[i]] + 1].displayName,
                score: scores[indexes[i]]
            });
        }
        return objects;
    }
    calculateMaxScores(scores, numBoxes, numClasses) {
        const maxes = [];
        const classes = [];
        for (let i = 0; i < numBoxes; i++) {
            let max = Number.MIN_VALUE;
            let index = -1;
            for (let j = 0; j < numClasses; j++) {
                if (scores[i * numClasses + j] > max) {
                    max = scores[i * numClasses + j];
                    index = j;
                }
            }
            maxes[i] = max;
            classes[i] = index;
        }
        return [maxes, classes];
    }
    async detect(img, maxNumBoxes = 20, minScore = 0.5) {
        return this.infer(img, maxNumBoxes, minScore);
    }
    dispose() {
        if (this.model != null) {
            this.model.dispose();
        }
    }
}
exports.ObjectDetection = ObjectDetection;
//# sourceMappingURL=coco.js.map