"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnifiFlows = void 0;
class UnifiFlows {
    constructor(unifi, config, endPointStyle, logger) {
        this.unifi = unifi;
        this.config = config;
        this.endpointStyle = endPointStyle;
        this.log = logger;
    }
    async enumerateCameras() {
        try {
            await this.ensureSessionIsValid();
            return await this.unifi.enumerateMotionCameras(this.session, this.endpointStyle);
        }
        catch (error) {
            throw new Error('ERROR: Could not enumerate motion sensors: ' + error);
        }
    }
    async assignMotionEventsToCameras(cameras) {
        try {
            await this.ensureSessionIsValid();
            const motionEvents = await this.unifi.getMotionEvents(this.session, this.endpointStyle);
            outer: for (const camera of cameras) {
                camera.lastMotionEvent = null;
                for (const motionEvent of motionEvents) {
                    if (camera.id === motionEvent.cameraId) {
                        if (motionEvent.score >= this.config.motion_score) {
                            this.log.debug('Unifi Motion event (' + motionEvent.id + ') accepted for camera: ' + camera.name + ' - Score: ' + motionEvent.score);
                            camera.lastMotionEvent = motionEvent;
                        }
                        else {
                            this.log.debug('Unifi Motion event (' + motionEvent.id + ') rejected for camera: ' + camera.name + ' - Score: ' + motionEvent.score);
                        }
                        continue outer;
                    }
                }
            }
            return cameras;
        }
        catch (error) {
            throw new Error('Could not detect motion: ' + error);
        }
    }
    async getCameraSnapshot(camera, width, height) {
        try {
            await this.ensureSessionIsValid();
            return this.unifi.getSnapshotForCamera(this.session, this.endpointStyle, camera, width, height);
        }
        catch (error) {
            throw new Error('Could not get camera snapshot: ' + error);
        }
    }
    async ensureSessionIsValid() {
        try {
            if (!this.unifi.isSessionStillValid(this.session, this.endpointStyle)) {
                this.session = await this.unifi.authenticate(this.config.username, this.config.password, this.endpointStyle);
            }
            return this.session;
        }
        catch (error) {
            throw new Error('Authentication failed: ' + JSON.stringify(error, null, 4));
        }
    }
}
exports.UnifiFlows = UnifiFlows;
//# sourceMappingURL=unifi-flows.js.map