"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageUtils = void 0;
const fs = require('fs');
const canvas_1 = require("canvas");
class ImageUtils {
    static async createImage(pathOrUrl) {
        try {
            return await (0, canvas_1.loadImage)(pathOrUrl);
        }
        catch (error) {
            throw new Error('Cannot load image!');
        }
    }
    static createCanvasFromImage(image) {
        const canvas = (0, canvas_1.createCanvas)(image.width, image.height);
        const ctx = canvas.getContext('2d');
        ctx.drawImage(image, 0, 0, image.width, image.height);
        return canvas;
    }
    static resizeCanvas(inputCanvas, targetWidth, targetHeight) {
        const canvas = (0, canvas_1.createCanvas)(targetWidth, targetHeight);
        const ctx = canvas.getContext('2d');
        if (inputCanvas) {
            ctx.drawImage(inputCanvas, 0, 0, inputCanvas.width, inputCanvas.height, 0, 0, targetWidth, targetHeight);
        }
        return canvas;
    }
    static async generateAnnotatedImage(image, detections) {
        const canvas = ImageUtils.createCanvasFromImage(image);
        const ctx = canvas.getContext('2d');
        for (const detection of detections) {
            ImageUtils.drawRect(ctx, detection.bbox);
            ImageUtils.drawText(ctx, detection);
        }
        return canvas;
    }
    static async remove(path) {
        return new Promise((resolve, reject) => {
            fs.unlink(path, () => {
                resolve();
            });
        });
    }
    static saveCanvasToFile(canvas) {
        return new Promise((resolve, reject) => {
            const snapshotName = 'snapshot-' + new Date().toISOString() + '.jpg';
            const out = fs.createWriteStream(this.userStoragePath + '/' + snapshotName);
            const stream = canvas.createJPEGStream({
                quality: 0.95,
                chromaSubsampling: false
            });
            stream.pipe(out);
            out.on('error', (error) => {
                reject('Cannot save image to disk: ' + error.message);
            });
            out.on('finish', () => {
                const fileName = this.userStoragePath + '/' + snapshotName;
                resolve(fileName);
            });
        });
    }
    static drawRect(ctx, bbox) {
        ctx.strokeStyle = 'rgba(255,0,0,1)';
        ctx.lineWidth = 2;
        ctx.beginPath();
        ctx.rect(bbox[0], bbox[1], bbox[2], bbox[3]);
        ctx.stroke();
    }
    static drawText(ctx, detection) {
        ctx.font = '16px Arial';
        ctx.fillStyle = 'red';
        ctx.fillText(detection.class + ': ' + Math.round(detection.score * 100) + '%', detection.bbox[0] + 5, detection.bbox[1] + 15);
    }
}
exports.ImageUtils = ImageUtils;
//# sourceMappingURL=image-utils.js.map