"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = void 0;
const https = __importStar(require("https"));
const node_fetch_1 = __importDefault(require("node-fetch"));
class Utils {
    static pause(duration) {
        return new Promise(res => setTimeout(res, duration));
    }
    static async retry(retries, fn, delay, retryCount = 0, lastError = null) {
        if (retryCount === retries) {
            throw lastError;
        }
        try {
            return Promise.resolve(await fn());
        }
        catch (e) {
            await this.pause(delay);
            return await this.retry(retries, fn, delay * 2, retryCount + 1, e);
        }
    }
    static async fetch(url, options, headers, networkLogger = this.fakeLogging()) {
        options.agent = this.httpsAgent;
        options.headers = headers;
        networkLogger.debug('Calling: ' + url);
        networkLogger.debug('Method: ' + options.method);
        networkLogger.debug('With headers: ' + JSON.stringify(options.headers, null, 4));
        if (options.body) {
            networkLogger.debug('Body: ' + JSON.stringify(options.body, null, 4));
        }
        let response = await (0, node_fetch_1.default)(url, options);
        networkLogger.debug('Response: \n' + JSON.stringify(response, null, 4));
        if (response.status === 401) {
            throw new Error('Invalid credentials');
        }
        if (response.status === 403) {
            throw new Error('Access Forbidden');
        }
        if (!response.ok) {
            throw new Error('Invalid response: ' + response);
        }
        return response;
    }
    static fakeLogging() {
        return {
            prefix: "FAKE_LOGGING",
            error(message, ...parameters) { },
            info(message, ...parameters) { },
            log(level, message, ...parameters) { },
            warn(message, ...parameters) { },
            debug(message, ...parameters) { },
        };
    }
}
exports.Utils = Utils;
Utils.httpsAgent = new https.Agent({
    rejectUnauthorized: false
});
//# sourceMappingURL=utils.js.map